@@ -34,6 +34,7 @@ from .const import LABEL_ONE_OFF
 from .const import LABEL_PROJECT
 from .const import LABEL_SERVICE
 from .const import LABEL_VERSION
+from .const import NANOCPUS_SCALE
 from .container import Container
 from .errors import HealthCheckFailed
 from .errors import NoHealthCheckConfigured
@@ -52,7 +53,10 @@ HOST_CONFIG_KEYS = [
     'cap_add',
     'cap_drop',
     'cgroup_parent',
+    'cpu_count',
+    'cpu_percent',
     'cpu_quota',
+    'cpus',
     'devices',
     'dns',
     'dns_search',
@@ -390,7 +394,7 @@ class Service(object):
             return containers
 
     def _execute_convergence_recreate(self, containers, scale, timeout, detached, start):
-            if len(containers) > scale:
+            if scale is not None and len(containers) > scale:
                 self._downscale(containers[scale:], timeout)
                 containers = containers[:scale]
 
@@ -408,14 +412,14 @@ class Service(object):
             for error in errors.values():
                 raise OperationFailedError(error)
 
-            if len(containers) < scale:
+            if scale is not None and len(containers) < scale:
                 containers.extend(self._execute_convergence_create(
                     scale - len(containers), detached, start
                 ))
             return containers
 
     def _execute_convergence_start(self, containers, scale, timeout, detached, start):
-            if len(containers) > scale:
+            if scale is not None and len(containers) > scale:
                 self._downscale(containers[scale:], timeout)
                 containers = containers[:scale]
             if start:
@@ -429,7 +433,7 @@ class Service(object):
                 for error in errors.values():
                     raise OperationFailedError(error)
 
-            if len(containers) < scale:
+            if scale is not None and len(containers) < scale:
                 containers.extend(self._execute_convergence_create(
                     scale - len(containers), detached, start
                 ))
@@ -448,7 +452,7 @@ class Service(object):
         )
 
     def execute_convergence_plan(self, plan, timeout=None, detached=False,
-                                 start=True, scale_override=None):
+                                 start=True, scale_override=None, rescale=True):
         (action, containers) = plan
         scale = scale_override if scale_override is not None else self.scale_num
         containers = sorted(containers, key=attrgetter('number'))
@@ -460,6 +464,11 @@ class Service(object):
                 scale, detached, start
             )
 
+        # The create action needs always needs an initial scale, but otherwise,
+        # we set scale to none in no-rescale scenarios (`run` dependencies)
+        if not rescale:
+            scale = None
+
         if action == 'recreate':
             return self._execute_convergence_recreate(
                 containers, scale, timeout, detached, start
@@ -793,6 +802,10 @@ class Service(object):
             init_path = options.get('init')
             options['init'] = True
 
+        nano_cpus = None
+        if 'cpus' in options:
+            nano_cpus = int(options.get('cpus') * NANOCPUS_SCALE)
+
         return self.client.create_host_config(
             links=self._get_links(link_to_self=one_off),
             port_bindings=build_port_bindings(
@@ -832,6 +845,9 @@ class Service(object):
             init=options.get('init', None),
             init_path=init_path,
             isolation=options.get('isolation'),
+            cpu_count=options.get('cpu_count'),
+            cpu_percent=options.get('cpu_percent'),
+            nano_cpus=nano_cpus,
         )
 
     def get_secret_volumes(self):
@@ -868,6 +884,7 @@ class Service(object):
             nocache=no_cache,
             dockerfile=build_opts.get('dockerfile', None),
             cache_from=build_opts.get('cache_from', None),
+            labels=build_opts.get('labels', None),
             buildargs=build_args
         )
 
